<?php
/**
 * @file
 *   Drush command to build a scaffolding around your site for building it.
 *
 *   It ensures that development modules (including devel) are downloaded, and
 *   it enables key site-building modules (such as Views UI).
 *
 *   You can copy this file to any of the following
 *     1. A .drush folder in your HOME folder.
 *     2. Anywhere in a folder tree below an active module on your site.
 *     3. /usr/share/drush/commands (configurable)
 *     4. In an arbitrary folder specified with the --include option.
 */

/**
 * Defalt projects to download from drupal.org.
 */
function scaffold_fetch() {
  $projects = 'content_type_overview devel enabled_modules examples node_export uuid xray';
  global $conf;

  // Override project defaults if they've been set in settings.php.
  if (isset($conf['scaffold_fetch'])) {
    return $conf['scaffold_fetch'];
  }

  // Add to project defaults if additions have been set in settings.php.
  if (isset($conf['scaffold_fetch_add'])) {
    $projects .= ' ' . $conf['scaffold_fetch_add'];
  }
  return $projects;
}

/**
 * Defalt modules to enable for the development copy of your site.
 *
 * Configuration thus far by scaffold is done with variable_set(). There does
 * not need to be a special way to override this, as using
 * $conf['variable_name'] = 'value'; in settings.php will work as always.
 */
function scaffold_enable() {
  $modules = 'devel styles_ui update views_ui';
  global $conf;

  // Override module defaults if they've been set in settings.php.
  if (isset($conf['scaffold_enable'])) {
    return $conf['scaffold_enable'];
  }

  // Add to module defaults if additions have been set in settings.php.
  if (isset($conf['scaffold_enable_add'])) {
    $modules .= ' ' . $conf['scaffold_enable_add'];
  }
  return $modules;
}

/**
 * Implements hook_drush_command().
/**
 * Implements hook_drush_command().
 *
 * @return
 *   An associative array describing commands.
 */
function scaffold_drush_command() {
  $items = array();

  // The 'scaffold-fetch' command.
  $items['scaffold-fetch'] = array(
    'description' => "Fetches development modules.",
    'arguments' => array(
      'projects' => 'A space-delimited list of drupal.org project names, with optional versions.  Defaults to ' . scaffold_fetch() . '.',
    ),
    'aliases' => array('sff'),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH, // No bootstrap at all.
  );

  // The 'scaffold-enable' command.
  $items['scaffold-enable'] = array(
    'description' => "Enables and configures certain site-building and development modules.",
    'arguments' => array(
      'modules' => 'A space-delimited list of modules, defaults to ' . scaffold_enable() . '.',
    ),
    'aliases' => array('sfe'),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
  );

  // The 'scaffold-build' command.
  $items['scaffold-build'] = array(
    'description' => "Fetches, enables, and configures certain site-building and development modules.",
    'arguments' => array(
      'projects' => 'A space-delimited list of drupal.org project names, with optional versions.  Defaults to ' . scaffold_fetch(),
    ),
    'options' => array(
      'enable' => 'A comma-delimited list of modules to enable, defaults to devel views_ui.',
    ),
    'aliases' => array('sfb'),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
  );

  return $items;
}

/**
 * Implementation of hook_drush_help().
 *
 * This function is called whenever a drush user calls
 * 'drush help <name-of-your-command>'. This hook is optional. If a command
 * does not implement this hook, the command's description is used instead.
 *
 * This hook is also used to look up help metadata, such as help
 * category title and summary.  See the comments below for a description.
 *
 * @param
 *   A string with the help section (prepend with 'drush:')
 *
 * @return
 *   A string with the help text for your command.
 */
function scaffold_drush_help($section) {
  switch ($section) {
    case 'drush:scaffold-fetch':
      return dt("Put devel projects in sites/all/modules/scaffold.");
    // The 'summary' meta item is displayed in `drush help --filter`,
    // and is used to give a general idea what the commands in this
    // command file do, and what they have in common.
    case 'drush:scaffold-enable':
      return dt("Enable development-aiding modules.");
    case 'drush:scaffold-build':
      return dt("Fetch projects and enable modules.");
    case 'meta:scaffold:summary':
      return dt("Provides site-buildig and developing tools that should be used in-site.");
  }
}

/**
 * Implementation of drush_hook_COMMAND_validate().
 *
 * The validate command should exit with
 * `return drush_set_error(...)` to stop execution of
 * the command.  In practice, calling drush_set_error
 * OR returning FALSE is sufficient.  See drush.api.php
 * for more details.
function drush_sandwich_make_me_a_sandwich_validate() {
  $name = posix_getpwuid(posix_geteuid());
  if ($name['name'] !== 'root') {
    return drush_set_error('MAKE_IT_YOUSELF', dt('What? Make your own sandwich.'));
  }
}
 */

/**
 * Drush command to fetch development & site building projects from drupal.org.
 *
 * Uses Git.  Haven't checked if it updates when possible instead of deleting
 * and cloning anew.
 *
 * The function name should be same as command name but with dashes turned to
 * underscores and 'drush_commandfile_' prepended, where 'commandfile' is
 * taken from the file 'commandfile.drush.inc', which in this case is 'sandwich'.
 * Note also that a simplification step is also done in instances where
 * the commandfile name is the same as the beginning of the command name,
 * "drush_example_example_foo" is simplified to just "drush_example_foo".
 * To also implement a hook that is called before your command, implement
 * "drush_hook_pre_example_foo".  For a list of all available hooks for a
 * given command, run drush in --debug mode.
 *
 * @see drush_invoke()
 * @see drush.api.php
 */
function drush_scaffold_fetch($projects = NULL) {
  if (!$projects) {
    $projects = scaffold_fetch();
  }
  $destination = '';
  if ($drupal_root = drush_get_context('DRUSH_DRUPAL_ROOT')) {
    $destination = $drupal_root . '/sites/all/modules/scaffolding';
    if (!is_dir($destination)) {
      drush_mkdir($destination);
    }
  }

  $result = drush_invoke_process_args(
    'pm-download',
    explode(' ', $projects),
    array(
      'package-handler' => 'git_drupalorg',
      'destination' => $destination,
    )
  );

  $msg = 'Fetched ' . $projects . '.';
  if ($result['output']) {
    $msg .= "\n" . $result['output'];
  }

  drush_print("\n" . $msg . "\n");
}

/**
 * Drush command to enable and configure development and site-building modules.
 */
function drush_scaffold_enable($modules = NULL) {
  $msg = 'Will enable ' . $modules . ' if not already enabled and configure useful starting settings related to them.';
  if (!$modules) {
    $modules = scaffold_enable();
  }
  $result = drush_invoke_process_args(
    'pm-enable',
    explode(' ', $modules)
  );
  // Print output from the invoked Drush function, if any.
  if ($result['output']) {
    $msg .= "\n" . $result['output'];
  }

  // Configure Views UI to use advanced settings.
  variable_set('views_ui_show_advanced_column', 1);
  variable_set('views_ui_show_listing_filters', 1);
  variable_set('views_ui_show_master_display', 1);

  // We need to clear caches before the user permission assignments will work
  // on the modules we just enabled.  Printing the result here is more a
  // development aid, and can be removed.
  $result = drush_invoke('cache-clear all');
  if ($result['output']) {
    $msg .= "\n" . $result['output'];
  }
  drush_print("\n" . $msg);

  // See devel output when using site as Anonymous or authenticated user.
  user_role_change_permissions(DRUPAL_ANONYMOUS_RID, array('access devel information' => TRUE));
  user_role_change_permissions(DRUPAL_AUTHENTICATED_RID, array('access devel information' => TRUE));
  drush_print("\nSet permissions to allow anonymous and normal authenticated users to see Devel module debug output.");
}

/**
 * Drush command to fetch and configure development and site-building aids.
 */
function drush_scaffold_build() {
  if ($enable_options = drush_get_option('enable')) {
    $enable_list = implode(' ', explode(',', $enable_options));
  }

  drush_scaffold_fetch();
  drush_scaffold_enable();
}
